# 2nd part of test of eval debugger command
# Use with example/eval.pl
set display eval dumper
set max width 300
set auto eval on
# Unlimited string length
set max string 0
list
s
print "--\n";
list
print "++\n";
s
s
s
s
s
s
s
bt
